@@ -79,6 +79,18 @@
 (if (= (difficulty) *hard*) (set-modtime mr-black *mod-spike* -1.))
 (set-acceleration mr-black 2.0)
 
+(define *min-value* 0.0)
+(define *max-value* 1.0)
+(define *starting-direction* 1.0)
+
+(define mr-black-animator
+  (animator
+     1.0 0.0 *starting-direction* *min-value* *max-value* *animator-bounce*
+     (lambda (value) (set-primary-color mr-black value value 0.0))
+     )
+  )
+
+
 ;(set-onoff mr-black #f)
 ;(set-modtime mr-black *mod-spike* -1.)
 ;(set-primary-color mr-black 0.5 1.0 0.5)
@@ -152,3 +164,6 @@
 ;(ply-x) (ply-y) (ply-z)
 ;(ply-dx) (ply-dy) (ply-dz)
 
+
+
+(sign "debug test" 1.0 15.0 0.0 10.5 10.5)
