@@ -1,5 +1,7 @@
-/* animated.cc
-   Use to create new .cc project files
+/** \file animated.cc
+	The base class for all Animated (ie. drawable) objects.
+*/
+/*
 
    Copyright (C) 2000  Mathias Broxvall
 
@@ -27,13 +29,17 @@ using namespace std;
 
 Animated::Animated() :GameHook() {
   Game::current->add(this);
+  zero(position);
   for(int i=0;i<3;i++) {
     specularColor[i] = 0.0;
     primaryColor[i] = 0.8;
     secondaryColor[i] = 0.3;
   }
   specularColor[3]=primaryColor[3]=secondaryColor[3]=1.0;
-  texture= 0;  // GLuint. 0, not -1
+  texture=0;
+  // Don't call any inherited version of the computeBoundBox yet to avoid problems
+  // with uninitialized variables.
+  Animated::computeBoundingBox();
 }
 Animated::~Animated() {
 }
@@ -42,7 +48,16 @@ void Animated::has_moved() {
 }
 void Animated::onRemove() {
   GameHook::onRemove();
-  Game::current->remove(this);    
+  Game::current->remove(this);
 }
 void Animated::draw() {}
 void Animated::draw2() {}
+void Animated::computeBoundingBox() {
+  /* Use a default size 2x2x2 boundingbox around object */
+  boundingBox[0][0]=-1.0;
+  boundingBox[0][1]=-1.0;
+  boundingBox[0][2]=-1.0;
+  boundingBox[1][0]=+1.0;
+  boundingBox[1][1]=+1.0;
+  boundingBox[1][2]=+1.0;
+}
