@@ -1,5 +1,7 @@
-/* baby.cc
-
+/** \file baby.cc
+   Small "baby" Mr Black, ie balls that can be killed by the player by rolling over them scoring a few points.
+*/
+/*
    Copyright (C) 2000  Mathias Broxvall
 
    This program is free software; you can redistribute it and/or modify
@@ -32,12 +34,15 @@ using namespace std;
 
 Baby::Baby(double x,double y) :Black(x,y) {
   acceleration = 1.0;
+  ballResolution=BALL_LORES;
   realRadius = 0.1; radius=realRadius;
   crashTolerance = 3;
   horizon=3.0;
   position[2] = Game::current->map->getHeight(position[0],position[1]) + radius;
   modTimeLeft[MOD_DIZZY] = -1.0;
   likesPlayer=-1;
+
+  setReflectivity(0.0,1);
 }
 
 void Baby::die() {
