@@ -45,18 +45,18 @@ extern GLuint hiresSphere;
 
 void Ball::init() {
   balls = new set<Ball *>();
-
   SDL_Surface *text;
   SDL_Color fgColor={255,255,255};
   GLfloat texcoord[4];
 
   text = TTF_RenderText_Blended(ingameFont, " ? ", fgColor);
-  dizzyTexture = LoadTexture(text, texcoord);
+  //dizzyTexture = LoadTexture(text, texcoord);
+  loadTexture("dizzy.png");
   SDL_FreeSurface(text);
-  dizzyTexMinX = texcoord[0];
-  dizzyTexMinY = texcoord[1];
-  dizzyTexMaxX = texcoord[2];
-  dizzyTexMaxY = texcoord[3];  
+  dizzyTexMinX = 0.0;
+  dizzyTexMinY = 0.0;
+  dizzyTexMaxX = 1.0;
+  dizzyTexMaxY = 1.0;    
 }
 void Ball::reset() {
   delete balls;
@@ -69,7 +69,7 @@ Ball::Ball() :Animated() {
   
   sink=0.0;
 
-  resolution = 6;
+  ballResolution=BALL_LORES;
   zero(position);
   zero(velocity);
   friction = 1.0;
@@ -96,8 +96,16 @@ Ball::Ball() :Animated() {
 
   identityMatrix(rotations);
   texture = loadTexture("ice.png");
+  nitroDebrisCount=0.0;
+
+  environmentTexture=0;
+  reflectivity=0.0;
+  metallic=0;
+  dontReflectSelf=0;
+}
+Ball::~Ball() { 
+  setReflectivity(0.0,0); // This effectivly deallocates all environment maps */
 }
-Ball::~Ball() {}
 
 /*********************************************
  draw() - Draws all the opaque parts of ball 
@@ -109,13 +117,16 @@ void Ball::draw() {
   double blend;
   double phase;
 
-  for(i=0;i<3;i++)
-	color[i] = primaryColor[i];
-  color[3] = 1.0;
+  // Fix when rendering environment map to not reflect oneself
+  if(dontReflectSelf) return;
 
-  specular[0] = specular[1] = specular[2] = 0.5;
+  for(i=0;i<3;i++) {
+	color[i] = primaryColor[i];
+	specular[i] = specularColor[i];
+  }
+  color[3] = 1.0;
   specular[3] = 1.0;
-  double shininess = 20.0;
+  double shininess = 10.0;
 
   glMatrixMode(GL_MODELVIEW);
   glPushAttrib(GL_ENABLE_BIT);
@@ -185,7 +196,60 @@ void Ball::draw() {
 	  glBindTexture(GL_TEXTURE_2D, texture);  
 	}
   }
-  gluSphere(qball, radius, resolution, resolution);
+
+  /*
+  specular[0] = 1.0;
+  specular[1] = 1.0;
+  specular[2] = 1.0;
+  glMaterialfv(GL_FRONT,GL_SPECULAR,specular);
+  glMaterialf(GL_FRONT,GL_SHININESS,10.0);	
+  */
+
+
+
+  //gluSphere(qball, radius, resolution, resolution);
+  glPushMatrix();
+  glEnable(GL_NORMALIZE);
+  glScalef(radius,radius,radius);
+  glCallList(sphereDisplayLists[ballResolution]);
+  glDisable(GL_NORMALIZE);
+  glPopMatrix();
+
+
+  
+  /* Draw reflection of environment */
+  if(Settings::settings->doReflections && reflectivity > 0.0 && environmentTexture) {
+	 //1 || (Game::current && (Ball*)Game::current->player1 == this && Settings::settings->gfx_details > 3)) {	
+	glPushAttrib(GL_ENABLE_BIT);
+	glDisable(GL_LIGHTING);
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
+	GLfloat c[4];
+	if(metallic) { c[0]=color[0]; c[1]=color[1]; c[2]=color[2]; c[3]=reflectivity; }
+	else { c[0]=1.0; c[1]=1.0; c[2]=1.0; c[3]=reflectivity; }
+	glColor4fv(c);
+	/*
+	glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,c);  
+	// TODO. Apply specular highlighting after environment textures
+	c[0]=1.0; c[1]=1.0; c[2]=1.0; c[3]=0.5;
+	glMaterialfv(GL_FRONT,GL_SPECULAR,c);
+	glMaterialf(GL_FRONT,GL_SHININESS,20.0);*/
+	glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
+	glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
+	glEnable(GL_TEXTURE_GEN_S);
+	glEnable(GL_TEXTURE_GEN_T);
+	//bindTexture("clouds.png");
+	glBindTexture(GL_TEXTURE_2D,environmentTexture);
+
+	//gluSphere(qball, radius, resolution, resolution);
+	glPushMatrix();
+	glEnable(GL_NORMALIZE);
+	glScalef(radius,radius,radius);
+	glCallList(sphereDisplayLists[ballResolution]);
+	glDisable(GL_NORMALIZE);
+	glPopMatrix();
+	glPopAttrib();
+  }
 
   if(modTimeLeft[MOD_SPIKE]) {
     glPushAttrib(GL_ENABLE_BIT);
@@ -308,6 +372,9 @@ void Ball::draw() {
 void Ball::draw2() {
   int i;
 
+  // Fix when rendering environment map to not reflect oneself
+  if(dontReflectSelf) return;
+
   if(!(modTimeLeft[MOD_EXTRA_LIFE] || modTimeLeft[MOD_JUMP] || modTimeLeft[MOD_DIZZY])) return;
 
   glMatrixMode(GL_MODELVIEW);
@@ -361,7 +428,7 @@ void Ball::draw2() {
 	glDisable(GL_CULL_FACE);
 	glDisable(GL_LIGHTING);
 	glEnable(GL_TEXTURE_2D);
-	glBindTexture(GL_TEXTURE_2D, dizzyTexture);
+	bindTexture("dizzy.png");
 	glColor4f(1.0,1.0,1.0,0.5);
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -389,6 +456,28 @@ void Ball::draw2() {
   glPopMatrix();
 }
 
+extern float fps;
+void Ball::doExpensiveComputations() {
+  /* TODO. Better test for when to render environment. Eg, only if ball is
+     visible etc. */
+  if(reflectivity > 0.0 && Settings::settings->doReflections) {
+    double dx=position[0] - MainMode::mainMode->camFocus[0];
+    double dy=position[1] - MainMode::mainMode->camFocus[1];
+    double dist=sqrt(dx*dx+dy*dy);
+    if(dist < 10) {
+      int update=20; /* How often to update environment map (on average) */
+      if(Settings::settings->gfx_details == 4) update = 10;
+      if(Settings::settings->gfx_details == 5) update = 5;   
+      if(((rand()>>7) % update) == 0 || 1) {
+	if(environmentTexture == 0) glGenTextures(1, &environmentTexture);
+	dontReflectSelf=1;
+		MainMode::mainMode->renderEnvironmentTexture(environmentTexture,position);
+		dontReflectSelf=0;
+      }
+    }
+  }
+}
+
 void Ball::tick(Real time) {
   int i;
   double phase;
@@ -403,6 +492,25 @@ void Ball::tick(Real time) {
 	}  
   }
 
+  /* Nitro balls create "debris" clouds constantly */
+  if(modTimeLeft[MOD_NITRO]) {
+	nitroDebrisCount += time;
+	while(nitroDebrisCount > 0.0) {
+	  nitroDebrisCount -= 0.25;
+	  Debris *d=new Debris(this,position,velocity,2.0+2.0*frandom());
+	  d->position[0] += (frandom()-0.5)*radius;
+	  d->position[1] += (frandom()-0.5)*radius;
+	  d->position[2] += radius*1.0;
+	  d->velocity[2] += 0.2;
+	  d->gravity=-0.1;
+	  d->modTimeLeft[MOD_GLASS]=-1.0;
+	  d->primaryColor[0]=0.1;
+	  d->primaryColor[1]=0.6;
+	  d->primaryColor[2]=0.1;
+	  d->no_physics=1;
+	}
+  }
+
   radius=realRadius;
   if(modTimeLeft[MOD_LARGE]) {
 	phase=min(modTimePhaseIn[MOD_LARGE]/5.0,1.0);
@@ -423,6 +531,16 @@ void Ball::tick(Real time) {
   physics(time);
 }
 
+void Ball::setReflectivity(double reflectivity,int metallic) {
+  this->reflectivity=reflectivity;
+  this->metallic=metallic;
+  if(reflectivity <= 0.0 && environmentTexture != 0) {
+	/* Deallocate environment texture */
+	glDeleteTextures(1,&environmentTexture);
+	environmentTexture=0;	
+  }  
+}
+
 Boolean Ball::physics(Real time) {
   Real x,y,h;
 
@@ -430,8 +548,11 @@ Boolean Ball::physics(Real time) {
   Map *map = Game::current->map;
 
   if(modTimeLeft[MOD_DIZZY]) {
-	rotation[0] += time * 7.0 * (frand((int) Game::current->gameTime + (int) this)-0.5); //sin(Game::current->gameTime*M_PI);
-	rotation[1] += time * 7.0 * (frand(47 + (int) Game::current->gameTime + (int) this)-0.5); //cos(Game::current->gameTime*M_PI);
+    /* Explanation. We cast the address of the ball into an integer in order to get
+       a unique random seed for every ball. Could possibly cause problems on 64 bit platforms, have to
+       look into it. */
+    rotation[0] += time * 7.0 * (frand((long) Game::current->gameTime + (long) this)-0.5); 
+    rotation[1] += time * 7.0 * (frand(47 + (long) Game::current->gameTime + (long) this)-0.5); 
   }
 
   /*  rotateX(-rotation[1]*time*2.0*M_PI*radius,rotations);
@@ -768,13 +889,13 @@ void Ball::generateSandDebris() {
   vel[0] = velocity[0] + 2.0*speed*sin(a); // + speed * 1/2048.0 * ((rand()%2048)-1024);
   vel[1] = velocity[1] + 2.0*speed*cos(a); // + speed * 1/2048.0 * ((rand()%2048)-1024);
   vel[2] = velocity[2]; // + speed * 1/2048.0 * ((rand()%2048)-1024);
-  Debris *d = new Debris(NULL,pos,vel);
+  Debris *d = new Debris(NULL,pos,vel,0.5+1.0*frandom());
+  d->initialSize=0.05;
   d->primaryColor[0] = 0.6 + 0.3*frandom();
   d->primaryColor[1] = 0.5 + 0.4*frandom();
   d->primaryColor[2] = 0.1 + 0.3*frandom();
-  d->life = 0.5 + 1.0 * frandom();
   d->friction = 0.0;
-  d->tick(0.0);
+  d->calcRadius();
 }
 void Ball::generateDebris(GLfloat color[4]) {
   Coord3d pos,vel;
@@ -784,13 +905,13 @@ void Ball::generateDebris(GLfloat color[4]) {
   vel[0] = velocity[0] + 2.0*speed*sin(a); // + speed * 1/2048.0 * ((rand()%2048)-1024);
   vel[1] = velocity[1] + 2.0*speed*cos(a); // + speed * 1/2048.0 * ((rand()%2048)-1024);
   vel[2] = velocity[2]; // + speed * 1/2048.0 * ((rand()%2048)-1024);
-  Debris *d = new Debris(NULL,pos,vel);
+  Debris *d = new Debris(NULL,pos,vel,0.5+1.0*frandom());
+  d->initialSize=0.05;
   d->primaryColor[0] = color[0];
   d->primaryColor[1] = color[1];
   d->primaryColor[2] = color[2];
-  d->life = 0.5 + 1.0 * frandom();
   d->friction = 0.0;
-  d->tick(0.0);
+  d->calcRadius();
 }
 
 void Ball::handleBallCollisions() {
@@ -835,6 +956,26 @@ void Ball::handleForcefieldCollisions() {
   for(;iter!=end;iter++) {
 	ForceField *ff = *iter;
 	if(!ff->is_on) continue;
+
+	/* Boundingbox test, continue if we cannot possibly be withing FF */
+	if(ff->direction[0] >= 0.0) {
+	  if(this->position[0] + radius < ff->position[0]) continue;
+	  if(this->position[0] - radius > ff->position[0]+ff->direction[0]) continue;
+	} else {
+	  if(this->position[0] + radius < ff->position[0]+ff->direction[0]) continue;
+	  if(this->position[0] - radius > ff->position[0]) continue;
+	}
+	if(ff->direction[1] >= 0.0) {
+	  if(this->position[1] + radius < ff->position[1]) continue;
+	  if(this->position[1] - radius > ff->position[1]+ff->direction[1]) continue;
+	} else {
+	  if(this->position[1] + radius < ff->position[1]+ff->direction[1]) continue;
+	  if(this->position[1] - radius > ff->position[1]) continue;
+	}
+	
+	/*                                */
+	/* Detailed collision computation */
+	/*                                */
 	Coord3d v,ff_normal;
 	sub(this->position,ff->position,v);		
 	v[2] = 0.0;
@@ -901,7 +1042,7 @@ void Ball::handleForcefieldCollisions() {
 		velocity[0] -= sign * bounce * ff_normal[0];
 		velocity[1] -= sign * bounce * ff_normal[1];
 	  }
-	}		
+	}
   }
 }
 void Ball::handleEdges() {
@@ -1001,7 +1142,6 @@ void Ball::handlePipes() {
 	} else {
 	  /* Ball is on the inside */
 	  inPipe=true;
-
 	  if(distance > pipe->radius*0.97 - radius && l != 0.0 && l != 1.0) {
 		/* Collision from inside */
 		Coord3d normal; assign(v1,normal); normalize(normal);
