@@ -1,6 +1,7 @@
-/* bird.cc
-   travel along their path and kill player if touched
-
+/** \file bird.cc
+   Travels along their path and kill player if touched.
+*/
+/*
    Copyright (C) 2000  Mathias Broxvall
    		       Yannick Perret
 
@@ -40,7 +41,7 @@ Bird::Bird(int x,int y,int dx,int dy,Real size,Real speed) {
   this->dy=dy-y;
   this->size=size;
   this->speed=speed;
-  this->lng = sqrt(this->dx*this->dx+this->dy*this->dy);
+  this->lng = sqrt((double)(this->dx*this->dx+this->dy*this->dy));
   position[0] = (Real)x+0.5;
   position[1] = (Real)y+0.5;  
   position[2] = Game::current->map->getHeight(position[0],position[1]) + .5;
