@@ -1,6 +1,7 @@
-/* black.cc
-   Represent the black hostile balls in the game
-
+/** \file black.cc
+   Represents the black hostile balls in the game
+*/
+/*
    Copyright (C) 2000  Mathias Broxvall
 
    This program is free software; you can redistribute it and/or modify
@@ -32,7 +33,7 @@ using namespace std;
 
 Black::Black(double x,double y) :Ball() {
   realRadius = 0.4; radius=realRadius;
-  resolution = 13;
+  ballResolution=BALL_HIRES;
   position[0] = x;
   position[1] = y;
   position[2] = Game::current->map->getHeight(position[0],position[1]) + radius;
@@ -43,11 +44,16 @@ Black::Black(double x,double y) :Ball() {
   primaryColor[0] = 0.0;
   primaryColor[1] = 0.0;
   primaryColor[2] = 0.2;
+  specularColor[0] = 1.0;
+  specularColor[1] = 1.0;
+  specularColor[2] = 1.0;
 
   bounceFactor=.8;
   acceleration = 4.0;
   horizon=5.0;
   likesPlayer=1;
+
+  setReflectivity(0.4,0);
 }
 void Black::die(int how) {
   int i,j;
@@ -65,7 +71,7 @@ void Black::die(int how) {
 		vel[0] = velocity[0] + 0.5 * 1/2048.0 * ((rand()%2048)-1024);
 		vel[1] = velocity[1] + 0.5 * 1/2048.0 * ((rand()%2048)-1024);
 		vel[2] = velocity[2] + 0.5 * 1/2048.0 * ((rand()%2048)-1024);
-		new Debris(this,pos,vel);
+		new Debris(this,pos,vel,2.0+8.0*frandom());
 	  }
 	position[2] += 1.0;
 	new ScoreSign(100,position);
