@@ -24,11 +24,10 @@
 
 using namespace std;
 
-Debris::Debris(Ball *ball,Coord3d pos,Coord3d vel) {
+Debris::Debris(Ball *ball,Coord3d pos,Coord3d vel,double dur) {
   int i;
   assign(pos,position);
   assign(vel,velocity);
-  life = 2.0 + 8.0 / 100.0 * (rand()%100);  
   if(Settings::settings->gfx_details <= GFX_DETAILS_NONE)
 	resolution = 4;
   else if(Settings::settings->gfx_details == GFX_DETAILS_SIMPLE) 
@@ -37,7 +36,6 @@ Debris::Debris(Ball *ball,Coord3d pos,Coord3d vel) {
 	resolution = 6;
   else if(Settings::settings->gfx_details >= GFX_DETAILS_EXTRA) 
 	resolution = 8;
-  realRadius=0.25; radius=realRadius;
   crashTolerance = 10000;
   if(ball) {
 	for(i=0;i<3;i++)
@@ -45,25 +43,32 @@ Debris::Debris(Ball *ball,Coord3d pos,Coord3d vel) {
 	for(i=0;i<NUM_MODS;i++)
 	  modTimeLeft[i] = ball->modTimeLeft[i];
 	modTimeLeft[MOD_DIZZY] = 0.0;
+	modTimeLeft[MOD_NITRO] = 0.0;
 	texture=ball->texture;
-  }
+  } 
+  if(ball) 
+	initialSize=ball->realRadius/4.0;
+  else 
+	initialSize=0.05;
+  endingSize=0.0;
+  duration=dur;
+  elapsedTime=0.0;
+  calcRadius();
+}
+void Debris::calcRadius() {
+  double a,b;
+  b=elapsedTime / duration;
+  a=1.0 - b;
+  realRadius = initialSize * a + endingSize * b;
+  radius=realRadius;
 }
 void Debris::tick(Real t) {
-  life -= t;
-
-  realRadius = 0.01 + life/10.0 * 0.2; 
-  radius=realRadius;
-
-  if(life < 0) remove(); //delete this;
+  elapsedTime += t;
+  calcRadius();
+  if(elapsedTime > duration) remove(); //delete this;
   else Ball::tick(t);
 }
-void Debris::draw() {
-  if(Settings::settings->gfx_details <= GFX_DETAILS_SIMPLE) {
-	resolution = 5;
-	Ball::draw();
-  } else	
-	Ball::draw();
-}
+void Debris::draw() { Ball::draw(); }
 void Debris::die(int how) {
   remove();
   //delete this;
