@@ -27,13 +27,16 @@
 
 class Debris : public Ball {
  public:
-  Debris(Ball*,Coord3d pos,Coord3d vel);
+  Debris(Ball*,Coord3d pos,Coord3d vel,double duration);
+  void calcRadius();         // Use this after modifying the parameters below
   void tick(Real);
   void draw();
   void die(int);
 
-  Real life;
+  int floatingUp;
+  double initialSize, endingSize, duration, elapsedTime;
  private:
+
 };
 
 #endif
