@@ -26,6 +26,7 @@ using namespace std;
 
 GameMode *GameMode::current;
 
+GameMode::GameMode() { keyUpReceived=1; }
 GameMode::~GameMode() {
 }
 void GameMode::activate(GameMode* gm) { 
@@ -35,6 +36,7 @@ void GameMode::activate(GameMode* gm) {
 }
 void GameMode::display(){}
 void GameMode::key(int){}
+void GameMode::keyUp(int){ keyUpReceived=1; }
 void GameMode::special(int key,int x,int y){}
 void GameMode::idle(Real td){}
 void GameMode::mouse(int state,int x,int y){}
