@@ -1,6 +1,7 @@
-/* general.h
+/** \file general.h
    Contains header information for *all* source files in the project.
-
+*/
+/*
    Copyright (C) 2000  Mathias Broxvall
 
    This program is free software; you can redistribute it and/or modify
@@ -18,19 +19,30 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <zlib.h>
 #include <math.h>
 #include <time.h>
+#include <iterator>
+#include <sys/time.h>
+#include "../gettext.h"
+#define _(String) gettext(String)
+
+#define GL_GLEXT_PROTOTYPES
+#include <GL/gl.h>
+#include <GL/glext.h>
 #include <GL/glu.h>
+
+
 //#include <g++-3/set.h>   //TODO: Fix configure.in
 #include <set>   //TODO: Fix configure.in
 #include <SDL/SDL.h>
 #include <SDL/SDL_mouse.h>
 #include <SDL/SDL_ttf.h>
 #include "libguile.h"
-#include "libguile/numbers.h" // Elliminate this line if it gives you any troubles
+#include "libguile/numbers.h" // Eliminate this line if it gives you any troubles
 #include <stdint.h>
 
 #ifdef __MINGW32__
@@ -51,6 +63,7 @@ extern char effectiveShareDir[];
 //#define SHARE_DIR "/home/matbr/development/trackballs/share"
 //#define SHARE_DIR "./share/trackballs"
 
+
 /*** More includes follows after typedeclarations ***/
 
 typedef unsigned char uchar;
@@ -80,14 +93,18 @@ double semiRand(int,int);
 double semiRand(int,int,int);
 double frandom();
 
+int dirExists(char *);
 int fileExists(char *);
+int pathIsFile(char *path); /** Checks if the given path points to a true file (not links) */
+int pathIsDir(char *path);  /** Checks if the given path points to a true directory (not links) */
+int pathIsLink(char *path); /** Checks if the given path points to a link */
 
-/*** Globals ***/
-extern SDL_Surface *screen;
-extern int silent;
-extern int low_memory; // if true, attempt to conserve memory
-extern int debug_joystick;
-extern int repair_joystick;
+/* Attempts to open a file (with given mode) in the following order:
+   1. In $HOME/.trackballs/
+   2. In ./
+   3. In configured share directory
+*/
+char *locateFile(char *basename,char mode);
 
 /* Sometimes these functions are already defined. If so, it is
    safe to remove them from below. */
@@ -98,5 +115,16 @@ double inline min(double a,double b) {return a<b?a:b;}
 double inline max(double a,double b) {return a>b?a:b;}
 */
 
+
 /* A modulus operations which handles negative results safly */
 int mymod(int v,int m);
+/* Returns the real time right now measured in seconds. Mostly usefull for debugging and optimizations */
+double getSystemTime();
+
+/*** Globals ***/
+extern SDL_Surface *screen;
+extern int silent;
+extern int low_memory; // if true, attempt to conserve memory
+extern int debug_joystick;
+extern int repair_joystick;
+
