@@ -518,7 +518,7 @@ SCM_DEFINE(set_bonus_level,"set-bonus-level",1,0,0,(SCM name),
   if(Game::current) {
 	char *sname = SCM_STRING_CHARS(name);
 	if(strlen(sname) > 0) {
-	  sprintf(Game::current->returnLevel,"%s",sname);
+	  snprintf(Game::current->returnLevel,sizeof(Game::current->returnLevel),"%s",sname);
 	  Game::current->map->isBonus=1;
 	} else
 	  Game::current->map->isBonus=0;
@@ -942,6 +942,7 @@ SCM_DEFINE (set_cell_flag, "set-cell-flag", 6, 0, 0, (SCM x0, SCM y0, SCM x1, SC
   for(int x=min(ix0,ix1);x<=max(ix0,ix1);x++)
 	for(int y=min(iy0,iy1);y<=max(iy0,iy1);y++) {
 	  Cell& c=Game::current->map->cell(x,y);
+	  c.displayListDirty=1;
 	  if(SCM_FALSEP(state))
 		c.flags = c.flags & (~iflag);
 	  else
@@ -967,6 +968,7 @@ SCM_DEFINE (set_cell_velocity, "set-cell-velocity", 6, 0, 0,
   for(int x=min(ix0,ix1);x<=max(ix0,ix1);x++)
 	for(int y=min(iy0,iy1);y<=max(iy0,iy1);y++) {
 	  Cell& c=Game::current->map->cell(x,y);
+	  c.displayListDirty=1;
 	  c.velocity[0] = scm_num2dbl(vx,FUNC_NAME);
 	  c.velocity[1] = scm_num2dbl(vy,FUNC_NAME);
 	}
@@ -997,6 +999,7 @@ SCM_DEFINE (set_cell_heights, "set-cell-heights", 8, 1, 0, (SCM x0, SCM y0, SCM
 	  c.heights[1] = scm_num2dbl(h1,FUNC_NAME);
 	  c.heights[2] = scm_num2dbl(h2,FUNC_NAME);
 	  c.heights[3] = scm_num2dbl(h3,FUNC_NAME);
+	  c.displayListDirty=1;
 	  if(SCM_NUMBERP(h4))
 		c.heights[4] = scm_num2dbl(h4,FUNC_NAME);
 	  else c.heights[4] = (c.heights[0] + c.heights[1] + c.heights[2] + c.heights[3]) / 4.;
@@ -1029,6 +1032,7 @@ SCM_DEFINE (set_cell_water_heights, "set-cell-water-heights", 8, 1, 0, (SCM x0,
 	  c.waterHeights[1] = scm_num2dbl(h1,FUNC_NAME);
 	  c.waterHeights[2] = scm_num2dbl(h2,FUNC_NAME);
 	  c.waterHeights[3] = scm_num2dbl(h3,FUNC_NAME);
+	  c.displayListDirty=1;
 	  if(SCM_NUMBERP(h4))
 		c.waterHeights[4] = scm_num2dbl(h4,FUNC_NAME);
 	  else c.waterHeights[4] = (c.waterHeights[0] + c.waterHeights[1] + c.waterHeights[2] + c.waterHeights[3]) / 4.;
@@ -1061,6 +1065,7 @@ SCM_DEFINE (set_cell_colors, "set-cell-colors", 8, 1, 0, (SCM x0, SCM y0, SCM x1
 	  c.colors[i][0] = (GLfloat) scm_num2dbl(r,FUNC_NAME);
 	  c.colors[i][1] = (GLfloat) scm_num2dbl(g,FUNC_NAME);
 	  c.colors[i][2] = (GLfloat) scm_num2dbl(b,FUNC_NAME);
+	  c.displayListDirty=1;
 	  if(SCM_NUMBERP(a))	  
 		c.colors[i][3] = (GLfloat) scm_num2dbl(a,FUNC_NAME);
 	  else
@@ -1094,6 +1099,7 @@ SCM_DEFINE (set_cell_wall_colors, "set-cell-wall-colors", 8, 1, 0, (SCM x0, SCM
 	  c.wallColors[i][0] = (GLfloat) scm_num2dbl(r,FUNC_NAME);
 	  c.wallColors[i][1] = (GLfloat) scm_num2dbl(g,FUNC_NAME);
 	  c.wallColors[i][2] = (GLfloat) scm_num2dbl(b,FUNC_NAME);
+	  c.displayListDirty=1;
 	  if(SCM_NUMBERP(a))	  
 		c.wallColors[i][3] = (GLfloat) scm_num2dbl(a,FUNC_NAME);
 	  else
@@ -1275,6 +1281,7 @@ SCM_GLOBAL_VARIABLE_INIT(s_mod_float,"*mod-float*",scm_long2num(MOD_FLOAT));
 SCM_GLOBAL_VARIABLE_INIT(s_mod_extra_life,"*mod-extra-life*",scm_long2num(MOD_EXTRA_LIFE));
 SCM_GLOBAL_VARIABLE_INIT(s_mod_large,"*mod-large*",scm_long2num(MOD_LARGE));
 SCM_GLOBAL_VARIABLE_INIT(s_mod_small,"*mod-small*",scm_long2num(MOD_SMALL));
+SCM_GLOBAL_VARIABLE_INIT(s_mod_nitro,"*mod-nitro*",scm_long2num(MOD_NITRO));
 
 SCM_GLOBAL_VARIABLE_INIT(s_easy,"*easy*",scm_long2num(0));
 SCM_GLOBAL_VARIABLE_INIT(s_normal,"*normal*",scm_long2num(1));
