@@ -1,6 +1,7 @@
-/* hofMode.h
+/** \file hofMode.cc
    Displayes the "Hall of Fame"
-
+*/
+/*
    Copyright (C) 2000  Mathias Broxvall
 
    This program is free software; you can redistribute it and/or modify
@@ -27,19 +28,22 @@
 #include "hofMode.h"
 #include "settings.h"
 #include "font.h"
+#include "menusystem.h"
 
 using namespace std;
 
 HallOfFameMode *HallOfFameMode::hallOfFameMode;
 SDL_Surface *HallOfFameMode::background;
 
+#define CODE_LEVELSET    1
+
 void HallOfFameMode::init() {
   char str[256];
   if(low_memory) background=NULL;
   else {
-	sprintf(str,"%s/images/displayHighscoreBackground.jpg",SHARE_DIR);
+	snprintf(str,sizeof(str),"%s/images/displayHighscoreBackground.jpg",SHARE_DIR);
 	background=IMG_Load(str);
-	if(!background) { printf("Error: Failed to load %s\n",str); exit(0); } 
+	if(!background) { printf(_("Error: Failed to load %s\n"),str); exit(0); } 
   }
   hallOfFameMode = new HallOfFameMode();
 }
@@ -48,9 +52,9 @@ void HallOfFameMode::activated() {
   char str[256];
 
   if(!background) {
-	sprintf(str,"%s/images/displayHighscoreBackground.jpg",SHARE_DIR);
+	snprintf(str,sizeof(str),"%s/images/displayHighscoreBackground.jpg",SHARE_DIR);
 	background=IMG_Load(str);
-	if(!background) { printf("Error: Failed to load %s\n",str); exit(0); } 
+	if(!background) { printf(_("Error: Failed to load %s\n"),str); exit(0); } 
   }
 
   /* Loads the background images. */
@@ -87,24 +91,35 @@ void HallOfFameMode::display() {
   glEnd();
   Leave2DMode();
 
-  int y=200,dy=40,size=24;
-  int x=350;
+  int y,dy,size,x,y2;
   if(screenWidth <= 640) x=300;
-  if(screenWidth >= 1024) x=450;
-  if(screenHeight <= 480) { y = 160; dy=32; size=16; }
-  if(screenHeight >= 768) { y = 280; dy=52; size=32; }
+  else if(screenWidth <= 800) x=350;
+  else if(screenWidth <= 1024) x=400;
+  else x=450;
+  if(screenHeight <= 480) { y = 160; y2 = 150; dy=32; size=16; }
+  else if(screenHeight <= 600) { y = 200; y2 = 180; dy=40; size=20; }
+  else if(screenHeight <= 768) { y = 280; y2 = 200; dy=52; size=24; }	
+  else { y = 300; y2 = 200; dy=58; size=24; }
 
+  int fontSize=size*2;
+  int col0 = screenWidth/2-x; //screenWidth/2 - 400;
+  int col1 = screenWidth/2+x+fontSize/2;
   Settings *settings = Settings::settings;
-  sprintf(str,"LevelSet: %s",settings->levelSets[levelSet].name);
+  clearSelectionAreas();
+  addText_Left(0,fontSize/2,y-dy*2,_("Level Set"),col0);
+  addText_Right(CODE_LEVELSET,fontSize/2,y-dy*2,settings->levelSets[levelSet].name,col1);
+
+  //sprintf(str,_("Level Set: %s"),settings->levelSets[levelSet].name);
   //TTF_SizeText(menuFont,str,&w,&h);
-  //draw2DString(menuFont,str,screenWidth/2 - w/2,y-dy,0,0,0);
-  Font::drawCenterSimpleText(0,str,screenWidth/2,(int)(y-dy-size*1.5),size*0.9,size,1.,1.,1.,1.);
+  //draw2DString(menuFont,str,screenWidth/2 - w/2,y-dy,0,0,0);  
+  //Font::drawCenterSimpleText(0,str,screenWidth/2,(int)(y-dy-size*1.5),size*0.9,size,1.,1.,1.,1.);
+
   HighScore *highscore = HighScore::highScore;
 
   for(int i=0;i<10;i++) {
 	//draw2DString(menuFont,&highscore->names[levelSet][i][0],screenWidth/2 - x,y+dy*i,220,220,64);
 	Font::drawSimpleText(0,&highscore->names[levelSet][i][0],screenWidth/2-x,y+dy*i-size,size*0.9,size,220/256.,220/256.,64/256.,1.0);
-	sprintf(str,"%d points",highscore->points[levelSet][i]);
+	snprintf(str,sizeof(str),_("%d points"),highscore->points[levelSet][i]);
 	Font::drawRightSimpleText(0,str,screenWidth/2+x+size/2,y+dy*i-size,size*0.9,size,220/256.,220/256.,64/256.,1.0);
 	//TTF_SizeText(menuFont,str,&w,&h);
 	//draw2DString(menuFont,str,screenWidth/2 + x - w,y+dy*i,220,220,64);
@@ -133,12 +148,11 @@ void HallOfFameMode::idle(Real td) {
   tickMouse(td);
 }
 void HallOfFameMode::mouseDown(int state,int mouseX,int mouseY) {
-  int y=200,dy=40;
-  if(screenHeight <= 480) { y = 160; dy=32; }
-  if(screenHeight >= 768) { y = 280; dy=52; }
-
-  if(mouseY < y-dy*1.5 && mouseY >= y-dy*2)
+  int selection=getSelectedArea();
+  if(selection == CODE_LEVELSET) {
 	levelSet = mymod((levelSet + (state==1?1:-1)),Settings::settings->nLevelSets);
-  else
+  } else {
 	isExiting=1;
+	timeLeft=1.0;
+  }
 }
