@@ -320,7 +320,8 @@ void MainMode::key(int key) {
 	}
     break;
   case statusPaused:
-    go_to_pause = 0;
+    if(key != 'p' && key != 'P')
+      go_to_pause = 0;
     //gameStatus = statusInGame;
     break;
   case statusInGame:
@@ -514,11 +515,14 @@ void MainMode::showInfo() {
   Player *player = Game::current->player1;
   int i,w,h;
 
+  /* Don't draw the panel if we have released the cursor etc. This is
+     usefull for screenshots. */
+  if(SDL_GetModState() & KMOD_CAPS) return;
+
+
   glColor3f(1.0,1.0,1.0);
-
-
   Enter2DMode();
-
+      
   // the panel
   bindTexture("left_panel.png");
   drawTextured2DRectangle(0,0,256,128);  
