@@ -1,8 +1,8 @@
 /* main.cc
    Contains the main of the game. See README for a description of the game.
 
-   Copyright (C) 2000  Mathias Broxvall
-   		       Yannick Perret
+   Copyright (C) 2000-2006  Mathias Broxvall
+              		    Yannick Perret
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ char *SHARE_DIR_DEFAULT=SHARE_DIR;
 using namespace std;
 
 /* Important globals */
-SDL_Surface *screen;
+SDL_Surface *screen = NULL;
 const char* program_name;
 int silent=0;
 int debug_joystick,repair_joystick;
@@ -89,7 +89,15 @@ void changeScreenResolution() {
   if(!screen) return;
   screenWidth = screen->w; screenHeight = screen->h;
   SDL_WarpMouse(screenWidth/2,screenHeight/2);	
-  SDL_WM_GrabInput(SDL_GRAB_ON);
+
+  /* Use CapsLock key to determine if mouse should be hidden+grabbed */
+  if(SDL_GetModState() & KMOD_CAPS) {
+    SDL_WM_GrabInput(SDL_GRAB_OFF);
+    SDL_ShowCursor(SDL_ENABLE);
+  } else {
+    SDL_WM_GrabInput(SDL_GRAB_ON);
+    SDL_ShowCursor(SDL_DISABLE);
+  }
 
   resetTextures();
 }
@@ -109,6 +117,14 @@ void print_usage (FILE* stream, int exit_code) {
   fprintf (stream, _("   -t  --touch           Updates a map to the latest format\n"));
   fprintf (stream, _("   -y  --low-memory      Attempt to conserve memory usage\n"));
   fprintf (stream, _("   -j  --repair-joystick Correct for bad joysticks\n"));
+  fprintf (stream, "\n");
+  fprintf (stream, _("Important keyboard shortcuts\n"));
+  fprintf (stream, _("   Escape     Soft quit\n"));
+  fprintf (stream, _("   CapsLock   Unhide mouse pointer\n"));
+  fprintf (stream, _("   CTRL-q     Quit the game immediatly\n"));
+  fprintf (stream, _("   CTRL-f     Toggle between fullscreen/windowed mode\n"));
+  fprintf (stream, _("   k          Kill the ball\n"));
+
   exit (exit_code);
 }
 
@@ -255,9 +271,17 @@ void innerMain(void *closure,int argc,char **argv) {
 
   // MB: Until here we are using 7 megs of memory
   changeScreenResolution();
+  if(!screen) {
+    printf(_("Could not initialize screen resolution.\nError message: '%s'\n"), SDL_GetError());
+	exit(-1);
+  }
   // MB: Until here we are using 42 megs of memory
+  
+  if(SDL_GetModState() & KMOD_CAPS) {
+    printf("Warning - capslock is on, the mouse will be visible and not grabbed\n");
+  }
+
 
-  SDL_ShowCursor(SDL_DISABLE);
   // set the name of the window
 
   double bootStart=((double)SDL_GetTicks()) /1000.0 ;
@@ -349,11 +373,15 @@ void innerMain(void *closure,int argc,char **argv) {
   /*                 */
 
 
-
-
   double oldTime,t,td;
   oldTime=((double)SDL_GetTicks()) /1000.0 ;
   SDL_WarpMouse(screenWidth/2,screenHeight/2);
+
+  /* Initialize random number generator */
+  int seed=(int) getSystemTime();
+  srand(seed);
+  int keyUpReceived=1;
+
   while(is_running) {
 
 	t=((double)SDL_GetTicks()) /1000.0 ;
@@ -409,18 +437,64 @@ void innerMain(void *closure,int argc,char **argv) {
 	  case SDL_MOUSEBUTTONDOWN:
 	    if(GameMode::current) GameMode::current->mouseDown(e->button,e->x,e->y);
 	    break;
-	  case SDL_KEYDOWN:
-	    if(event.key.keysym.sym == 'q') exit(0);
+	  case SDL_KEYUP:
+	    /* Prevent repeated keys */
+	    keyUpReceived=1;
 
-		if(event.key.keysym.sym == SDLK_ESCAPE) {
-		  if(editMode) {
-			((EditMode*)GameMode::current)->askQuit();
-		  } else if((GameMode::current && GameMode::current == MenuMode::menuMode))
-			is_running=0;
-		  else GameMode::activate(MenuMode::menuMode);
-		}
-		else if(GameMode::current) GameMode::current->key(event.key.keysym.sym);
-		break;
+	    /* Use CapsLock key to determine if mouse should be hidden+grabbed */
+	    if(event.key.keysym.sym == SDLK_CAPSLOCK) {	  
+	      if(SDL_GetModState() & KMOD_CAPS) {
+		SDL_WM_GrabInput(SDL_GRAB_OFF);
+		SDL_ShowCursor(SDL_ENABLE);
+	      } else {
+		SDL_WM_GrabInput(SDL_GRAB_ON);
+		SDL_ShowCursor(SDL_DISABLE);
+	      }
+	    } else
+	      GameMode::current->keyUp(event.key.keysym.sym);
+	  case SDL_KEYDOWN:
+
+	    /* Always quit if the 'q' key is pressed */
+	    if(event.key.keysym.sym == 'q' && SDL_GetModState() & KMOD_CTRL) exit(0);
+
+	    /* Change between fullscreen/windowed mode if the 'f' key
+	       is pressed */
+	    else if(event.key.keysym.sym == 'f' && SDL_GetModState() & KMOD_CTRL) {
+	      Settings::settings->is_windowed = Settings::settings->is_windowed ? 0 : 1;
+	      changeScreenResolution();
+	      /* Flush all events that occured while switching screen
+		 resolution */
+	      while(SDL_PollEvent(&event)) {}	      
+	    }
+
+	    /* Use CapsLock key to determine if mouse should be hidden+grabbed */
+	    else if(event.key.keysym.sym == SDLK_CAPSLOCK) {
+	      if(SDL_GetModState() & KMOD_CAPS) {
+		SDL_WM_GrabInput(SDL_GRAB_OFF);
+		SDL_ShowCursor(SDL_ENABLE);
+	      } else {
+		SDL_WM_GrabInput(SDL_GRAB_ON);
+		SDL_ShowCursor(SDL_DISABLE);
+	      }
+	    }
+
+	    else if(event.key.keysym.sym == SDLK_ESCAPE) {
+	      if(editMode) {
+		((EditMode*)GameMode::current)->askQuit();
+	      } else if((GameMode::current && GameMode::current == MenuMode::menuMode))
+		is_running=0;
+	      else { GameMode::activate(MenuMode::menuMode); while(SDL_PollEvent(&event)) {} }
+
+	    }
+	    else if(GameMode::current) {
+	      /* Prevent repeated keys */
+	      if(!keyUpReceived) break;
+	      keyUpReceived=0;
+
+	      GameMode::current->key(event.key.keysym.sym);
+	    }
+
+	    break;
 	  }
 	}
   }
@@ -516,8 +590,13 @@ int main(int argc,char **argv) {
 
   /* Initialialize i18n with system defaults, before anything else (eg. settings) have been loaded */
   setlocale(LC_ALL, ""); 
-  char localedir[256];
-  snprintf(localedir,255,"%s/locale",effectiveShareDir);
+  char localedir[512];
+  
+#ifdef LOCALEDIR
+  snprintf(localedir,511,"%s",LOCALEDIR);
+#else
+  snprintf(localedir,511,"%s/locale",effectiveShareDir);
+#endif
   bindtextdomain(PACKAGE,localedir);
   textdomain(PACKAGE);
 
