@@ -33,6 +33,7 @@ ScoreSign::ScoreSign(int points,Coord3d position)
   this->points = 0.0;
   pointsLeft = points;
   pointsPerSecond = points / 4.0;
+  lastLife=life + 1.0;
 }
 
 void ScoreSign::tick(Real t) {
@@ -42,11 +43,15 @@ void ScoreSign::tick(Real t) {
   points = pointsPerSecond * (4.0 - life + t);
   int p = ((int) points) - oldPoints;
 
-  if(points >= 0)
-    sprintf(str,"+%d",(int) points);
-  else
-    sprintf(str,"%d",(int) points);
-  mkTexture(str);
+  /* Limit number of recreated textures to 10 textures/second */
+  if(life < lastLife - 0.1) {
+	lastLife=life;
+	if(points >= 0)
+	  snprintf(str,sizeof(str),"+%d",(int) points);
+	else
+	  snprintf(str,sizeof(str),"%d",(int) points);
+	mkTexture(str);
+  }
   if(Game::current)
 	Game::current->player1->score += p;
 
