@@ -35,6 +35,7 @@
 #include "hofMode.h"
 #include "font.h"
 #include "menusystem.h"
+#include "sound.h"
 
 #define SETUP_NAME   0
 #define NUM_FIELDS   1
@@ -71,7 +72,11 @@ void SetupMode::init() {
 
   setupMode=new SetupMode();
 }
-SetupMode::SetupMode() {gamer=new Gamer();levelSet=level=0;}
+SetupMode::SetupMode() :GameMode() {
+  gamer=new Gamer();
+  levelSet=level=0;  
+  screenshot=0;  
+}
 void SetupMode::display() {
   glPushAttrib(GL_ENABLE_BIT);
   glClearColor(0.0,0.0,0.0,0.0);
@@ -80,7 +85,10 @@ void SetupMode::display() {
   /* Draw background */
   Enter2DMode();
   glColor4f(1.0,1.0,1.0,1.0);
-  glBindTexture(GL_TEXTURE_2D, texture);
+  if(screenshot)
+    glBindTexture(GL_TEXTURE_2D, screenshot);
+  else
+    glBindTexture(GL_TEXTURE_2D, texture);
   glBegin(GL_TRIANGLE_STRIP);
   glTexCoord2f(texMaxX, texMinY); glVertex2i(0,   0  );
   glTexCoord2f(texMinX, texMinY); glVertex2i(screenWidth, 0  );
@@ -100,6 +108,50 @@ void SetupMode::display() {
   int row0 = screenHeight/2 - 100;
   int rowSep = fontSize+5;
 
+  int INFO_BASE=screenHeight - 70;
+  int INFO_DELTA=20;
+  int INFO_RIGHT=20;
+  double INFO_WIDTH=10.0, INFO_HEIGHT=10.0, INFO_R=1.0, INFO_G=1.0, INFO_B=1.0, INFO_A=1.0;
+
+  double DESC_WIDTH=14.0, DESC_HEIGHT=14.0, DESC_R=1.0, DESC_G=1.0, DESC_B=1.0, DESC_A=1.0;
+
+  /* Special fix for lower screen resolutions */
+  if(screenWidth == 640) {
+    fontSize=32;
+    titleFontSize=48;
+    col0 = screenWidth/2 - 300;
+    col1 = screenWidth/2 + 50;
+    rowSep=fontSize+5;
+
+    INFO_WIDTH=8;
+    INFO_HEIGHT=8;
+    INFO_DELTA=16;
+
+    DESC_WIDTH=8.0;
+    DESC_HEIGHT=8.0;
+  }
+  if(screenWidth == 800) {
+    fontSize=32;
+    titleFontSize=48;
+    col0 = screenWidth/2 - 350;
+    col1 = screenWidth/2 + 75;
+    rowSep=fontSize+5;
+
+    DESC_WIDTH=10.0;
+    DESC_HEIGHT=10.0;
+
+    /*
+    INFO_WIDTH=8;
+    INFO_HEIGHT=8;
+    INFO_DELTA=16;    */
+  }
+
+
+  int DESC_RIGHT=(int) (col0 - DESC_WIDTH);
+  int DESC_BASE=(int) (row0 + rowSep*4);
+  int DESC_DELTA=(int) (DESC_HEIGHT*2);
+
+
   /* Draw title */
   addText_Center(0,titleFontSize/2,64,_("Setup new game"),screenWidth/2);
 
@@ -124,16 +176,16 @@ void SetupMode::display() {
 
   /* Difficulty */
   addText_Left(0,fontSize/2,row0+rowSep*3,_("Difficulty"),col0);
-  addText_Left(CODE_DIFFICULTY,fontSize/2,row0+rowSep*3,(settings->difficulty==0?_("Easy"):(settings->difficulty==1?_("Normal"):_("Hard"))),col1);
+  addText_Left(CODE_DIFFICULTY,fontSize/2,row0+rowSep*3,
+	       (settings->sandbox?
+		(settings->difficulty==0?_("Sandbox (Easy)"):
+		 (settings->difficulty==1?_("Sandbox (Normal)"):_("Sandbox (Hard)"))):
+		(settings->difficulty==0?_("Easy"):(settings->difficulty==1?_("Normal"):_("Hard")))),col1);
 
   /* Start */
   addText_Center(CODE_START,fontSize/2,screenHeight-64,_("Start Game"),screenWidth/2);
 
   /* Info */
-  int INFO_BASE=screenHeight - 70;
-  int INFO_DELTA=20;
-  int INFO_RIGHT=20;
-  double INFO_WIDTH=10.0, INFO_HEIGHT=10.0, INFO_R=1.0, INFO_G=1.0, INFO_B=1.0, INFO_A=1.0;
   snprintf(str,sizeof(str),_("Total games played: %d"), gamer->timesPlayed);
   Font::drawSimpleText(0, str, INFO_RIGHT, INFO_BASE+INFO_DELTA*0, INFO_WIDTH, INFO_HEIGHT, INFO_R, INFO_G, INFO_B, INFO_A); 
   if(gamer->timesPlayed)
@@ -144,6 +196,13 @@ void SetupMode::display() {
   snprintf(str,sizeof(str),_("Levels completed: %d"), gamer->nLevelsCompleted);
   Font::drawSimpleText(0, str, INFO_RIGHT, INFO_BASE+INFO_DELTA*2, INFO_WIDTH, INFO_HEIGHT, INFO_R, INFO_G, INFO_B, INFO_A); 
 
+  /* Descriptive level text */
+  int lineno;
+  for(lineno=0;lineno<5;lineno++) 
+    Font::drawSimpleText(0, settings->levelSets[levelSet].description[lineno],
+			 DESC_RIGHT, DESC_BASE+DESC_DELTA*lineno, DESC_WIDTH, DESC_HEIGHT, 
+			 DESC_R, DESC_G, DESC_B, DESC_A);
+
 
   /*                      */
   /* Draw the player ball */
@@ -218,7 +277,7 @@ void SetupMode::key(int key) {
 	name=-1;
 	return;
   }
-  if(len == 19) /* TODO. Beep */ return;
+  if(len == 19) { playEffect(SFX_PLAYER_DIES); return; }
   if(isprint(key)) {
 	if(SDL_GetModState() & (KMOD_LSHIFT | KMOD_RSHIFT))
 	  key = toupper(key);
@@ -244,7 +303,7 @@ void SetupMode::idle(Real td) {
 }
 void SetupMode::activated() {  
   char str[256];
- 
+
   gamer->reloadNames();
 
   if(!background) {
@@ -262,12 +321,22 @@ void SetupMode::activated() {
   texMaxY = texcoord[3];
 
   t = 0.0; level=0; name=-1;
+
+  levelSetChanged();
 }
 void SetupMode::deactivated() {
-  glDeleteTextures(1,&texture); gamer->save();
+  if(texture != 0) {
+    glDeleteTextures(1,&texture); 
+    texture=0;
+  }
+  if(screenshot) {
+    glDeleteTextures(1,&screenshot); 
+    screenshot=0;
+  }
+  gamer->save();
   if(low_memory && background) {
-	SDL_FreeSurface(background);
-	background=NULL;
+    SDL_FreeSurface(background);
+    background=NULL;
   }
 }
 void SetupMode::start() {
@@ -302,7 +371,9 @@ void SetupMode::mouseDown(int button,int x,int y) {
 		{ gamer->color = 4; gamer->textureNum = (gamer->textureNum+1)%numTextures; }
 	  break;
 	case CODE_DIFFICULTY: 
-	  settings->difficulty = mymod((settings->difficulty+(left?1:-1)),3); 
+	  settings->difficulty = mymod((settings->difficulty+3*settings->sandbox+(left?1:-1)),6); 
+	  settings->sandbox = settings->difficulty / 3;
+	  settings->difficulty = settings->difficulty % 3;
 	  break;
 	case CODE_START_LV: 
 	  level = mymod((level+(left?1:-1)),gamer->nKnownLevels[levelSet]);
@@ -317,7 +388,29 @@ void SetupMode::mouseDown(int button,int x,int y) {
 	  break;
 	case CODE_LEVEL_SET: 
 	  levelSet = mymod((levelSet+(left?1:-1)),settings->nLevelSets); 
-	  level=0; break;
+	  level=0; 
+	  levelSetChanged();
+	  break;
 	}
   } 
 }
+
+void SetupMode::levelSetChanged() {
+  Settings *settings=Settings::settings;
+
+  if(screenshot != 0) {
+    glDeleteTextures(1,&screenshot);
+    screenshot=0;
+  }
+  
+  if(settings->levelSets[levelSet].imagename[0] != 0 && settings->gfx_details >= 2) {
+    SDL_Surface *surf=IMG_Load(settings->levelSets[levelSet].imagename);
+    if(surf) {
+      GLfloat texcoord[4];
+      screenshot = LoadTexture(surf,texcoord);
+      SDL_FreeSurface(surf);
+      screenshotMax[0]=texcoord[2];
+      screenshotMax[1]=texcoord[3];
+    }
+  }
+}
