@@ -35,6 +35,8 @@ class SetupMode : public GameMode {
   static SetupMode *setupMode;
  private:
   void start();
+  void levelSetChanged();
+
   class Gamer *gamer;
   int levelSet,level,name;
   static SDL_Surface *background;
@@ -43,6 +45,9 @@ class SetupMode : public GameMode {
   GLuint texture;
   GLfloat texMinX, texMinY;
   GLfloat texMaxX, texMaxY;
+
+  GLuint screenshot;
+  GLfloat screenshotMax[2];
 };
 
 #endif
